/*
 * This program is based on gllin.c of project openmoko-agps with slight modification.
 *
 *  You can find this project here:
 *  http://projects.openmoko.org/projects/openmoko-agpsui/ * 
 */

#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <termios.h>

#include "ubx.h"
#include "agps-onlinec.h"

#define UBX_DEBUG 		0

extern struct agps_onlinec_data agps;

int dev_open(void)
{
		struct termios tio;
	
    	agps.fd = open(pipeName, O_RDWR);
			
		if (tcgetattr(agps.fd, &tio) < 0){
			printf("can't get attribute \n");
			return 1;
		}

		tio.c_cflag = B9600 | CS8 | CLOCAL | CREAD;
		tio.c_lflag &= ~ICANON;
		tio.c_lflag &= ~ECHO;

		tcflush(agps.fd, TCIFLUSH);
      	tcsetattr(agps.fd,TCSANOW,&tio);
	
		return 0;	
}

static ssize_t bwrite(const unsigned char *buf, size_t count, int delay)
{
	size_t i;
	
	if (delay)
	{
		for (i = 0; i < count; i++)
		{
			if (write(agps.fd, buf + i, 1) != 1)
				return -1;
			usleep(delay * 1000);
		}
	}
	else
			return write(agps.fd, (const unsigned char *) buf, count);	

	return i;
}

int ubx_raw_write(const unsigned char *raw_data, int size)
{	
	if (bwrite(raw_data, size, 0) != size){
		printf("ubx failed \n");
		return 1;
	}

	return 0;
}

static void ubx_send_msg(char cls, char id, const unsigned char *ubx, int size, int delay)
{
	unsigned char header[6], checksum[2];
	int i;

	header[0] = 0xb5;
	header[1] = 0x62;
	header[2] = cls;
	header[3] = id;
	header[4] = size & 0xff;
	header[5] = (size >> 8) & 0xff;

	checksum[0] = checksum[1] = 0;
	for (i = 2; i < 6; i++)
	{
		checksum[0] += header[i];
		checksum[1] += checksum[0];
	}

	for (i = 0; i < size; i++)
	{
		checksum[0] += ubx[i];
		checksum[1] += checksum[0];
	}

	if (bwrite(header, 6, delay) != 6)
		goto fail;

	if (size)
	{
		if (bwrite((void *) ubx, size, delay) != size)
			goto fail;
	}

	if (bwrite(checksum, 2, delay) != 2)
		goto fail;

	if (UBX_DEBUG)
	{
		printf("ubx sent: ");
		for (i = 0; i < 6; i++)
			printf("%02x ", header[i]);
		if (size)
		{
			for (i = 0; i < size; i++)
				printf("%02x ", ubx[i]);
		}

		printf("%02x %02x\n", checksum[0], checksum[1]);
	}

	return;
fail:
	printf("ubx failed\n");
}

void gps_reset(void)
{
	unsigned char ubx[4];
	int bbr = 0x7ff;

	ubx[0] = bbr & 0xff;
	ubx[1] = (bbr >> 8) & 0xff;
	ubx[2] = 0x2; /* Resetting GPS */
	ubx[3] = 0x0;

	ubx_send_msg(0x06, 0x04, ubx, 4, 4);	
	sleep(2);
}

